@@ -1,5 +1,5 @@
 /* Definitions of target machine for GNU compiler, for HP-UX.
-   Copyright (C) 1991, 1995, 1996, 2002, 2003, 2004, 2007
+   Copyright (C) 1991, 1995, 1996, 2002, 2003, 2004, 2007, 2008, 2009, 2010
    Free Software Foundation, Inc.
 
 This file is part of GCC.
@@ -22,6 +22,9 @@ along with GCC; see the file COPYING3.  If not see
 #undef TARGET_HPUX
 #define TARGET_HPUX 1
 
+#undef HPUX_LONG_DOUBLE_LIBRARY
+#define HPUX_LONG_DOUBLE_LIBRARY 1
+
 #undef TARGET_DEFAULT
 #define TARGET_DEFAULT MASK_BIG_SWITCH
 
@@ -33,7 +36,6 @@ along with GCC; see the file COPYING3.  If not see
 #define PTRDIFF_TYPE "int"
 
 #define LONG_DOUBLE_TYPE_SIZE 128
-#define HPUX_LONG_DOUBLE_LIBRARY
 #define FLOAT_LIB_COMPARE_RETURNS_BOOL(MODE, COMPARISON) ((MODE) == TFmode)
 
 /* GCC always defines __STDC__.  HP C++ compilers don't define it.  This
@@ -56,11 +58,11 @@ along with GCC; see the file COPYING3.  If not see
 	builtin_define ("__hpux__");				\
 	builtin_define ("__unix");				\
 	builtin_define ("__unix__");				\
+	builtin_define ("__STDC_EXT__");			\
 	if (c_dialect_cxx ())					\
 	  {							\
 	    builtin_define ("_HPUX_SOURCE");			\
 	    builtin_define ("_INCLUDE_LONGLONG");		\
-	    builtin_define ("__STDC_EXT__");			\
 	  }							\
 	else if (!flag_iso)					\
 	  {							\
@@ -76,8 +78,6 @@ along with GCC; see the file COPYING3.  If not see
 		builtin_define ("_PWB");			\
 		builtin_define ("PWB");				\
 	      }							\
-	    else						\
-	      builtin_define ("__STDC_EXT__");			\
 	  }							\
 	if (TARGET_SIO)						\
 	  builtin_define ("_SIO");				\
@@ -116,12 +116,4 @@ along with GCC; see the file COPYING3.  If not see
 #undef TARGET_HPUX_UNWIND_LIBRARY
 #define TARGET_HPUX_UNWIND_LIBRARY 1
 
-/* Handle #pragma weak and #pragma pack.  */
-#undef HANDLE_SYSV_PRAGMA
-#define HANDLE_SYSV_PRAGMA
-
-/* Define this so we can compile MS code for use with WINE.  */
-#undef HANDLE_PRAGMA_PACK_PUSH_POP
-#define HANDLE_PRAGMA_PACK_PUSH_POP
-
 #define MD_UNWIND_SUPPORT "config/pa/hpux-unwind.h"
