@@ -1,5 +1,6 @@
 /* Definitions for target OS TPF for GNU compiler, for IBM S/390 hardware
-   Copyright (C) 2003, 2004, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2004, 2005, 2007, 2009,
+   2010 Free Software Foundation, Inc.
    Contributed by P.J. Darcy (darcypj@us.ibm.com),
                   Hartmut Penner (hpenner@de.ibm.com), and
                   Ulrich Weigand (uweigand@de.ibm.com).
@@ -124,6 +125,6 @@ along with GCC; see the file COPYING3.  If not see
 #define MD_UNWIND_SUPPORT "config/s390/tpf-unwind.h"
 
 /* IBM copies these libraries over with these names.  */
-#define MATH_LIBRARY "-lCLBM"
-#define LIBSTDCXX "-lCPP1"
+#define MATH_LIBRARY "CLBM"
+#define LIBSTDCXX "CPP1"
 #endif /* ! _TPF_H */
