@@ -1,5 +1,5 @@
 /* General Solaris system support.
-   Copyright (C) 2004, 2005 , 2007, 2010 Free Software Foundation, Inc.
+   Copyright (C) 2004, 2005 , 2007, 2010, 2011 Free Software Foundation, Inc.
    Contributed by CodeSourcery, LLC.
 
 This file is part of GCC.
@@ -26,7 +26,7 @@ along with GCC; see the file COPYING3.  If not see
 #include "tm.h"
 #include "rtl.h"
 #include "tm_p.h"
-#include "toplev.h"
+#include "diagnostic-core.h"
 #include "ggc.h"
 
 tree solaris_pending_aligns, solaris_pending_inits, solaris_pending_finis;
@@ -134,10 +134,8 @@ solaris_assemble_visibility (tree decl ATTRIBUTE_UNUSED,
      STV_SINGLETON, and STV_ELIMINATE.
 
      See Linker and Libraries Guide, Ch. 2, Link-Editor, Defining
-     Additional Symbols with a mapfile,
-     http://docs.sun.com/app/docs/doc/819-0690/gdzmc?a=view
-     and Ch. 7, Object-File Format, Symbol Table Section,
-     http://docs.sun.com/app/docs/doc/819-0690/chapter6-79797?a=view  */
+     Additional Symbols, and Ch. 7, Object-File Format, Symbol Table
+     Section.  */
 
   static const char * const visibility_types[] = {
     NULL, "symbolic", "hidden", "hidden"
