@@ -4,8 +4,8 @@
 extern void *alloca (__SIZE_TYPE__);
 
 void big (void);
-inline void *q (void)
-{				/* { dg-warning "(function not inlinable|alloca)" } */
+inline void *q (void) /* { dg-warning "(function not inlinable|alloca)" } */
+{
 	return alloca (10);
 }
 inline void *t (void)
