@@ -1,5 +1,7 @@
 /* Contributed by Nicola Pero - Fri Mar  9 19:39:15 CET 2001 */
-#include <objc/objc.h>
+
+#include <stdlib.h>
+#include "../../objc-obj-c++-shared/next-mapping.h"
 #include <objc/objc-api.h>
 
 /* Test the hidden argument _cmd to method calls */
@@ -12,10 +14,13 @@
 @end
 
 @implementation TestClass
-+ (const char*) method;
++ (const char*) method
 {
   return sel_get_name (_cmd);
 }
+#ifdef __NEXT_RUNTIME__
++ initialize { return self; }
+#endif
 @end
 
 
