@@ -1,6 +1,7 @@
 /* Contributed by Nicola Pero -  Wed Dec  5 17:12:40 GMT 2001 */
+#include <stdlib.h>
+#import "../../objc-obj-c++-shared/Object1.h"
 #include <objc/objc.h>
-#include <objc/Object.h>
 
 typedef enum { black, white } color;
 
@@ -49,3 +50,4 @@ int main (void)
 
   return 0;
 }
+#include "../../objc-obj-c++-shared/Object1-implementation.h"
