@@ -1,11 +1,12 @@
 // std::moneypunct implementation details, generic version -*- C++ -*-
 
-// Copyright (C) 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
+// Copyright (C) 2001, 2002, 2003, 2004, 2005, 2009, 2010
+// Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2, or (at your option)
+// Free Software Foundation; either version 3, or (at your option)
 // any later version.
 
 // This library is distributed in the hope that it will be useful,
@@ -13,19 +14,14 @@
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 
-// You should have received a copy of the GNU General Public License along
-// with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-// USA.
+// Under Section 7 of GPL version 3, you are granted additional
+// permissions described in the GCC Runtime Library Exception, version
+// 3.1, as published by the Free Software Foundation.
 
-// As a special exception, you may use this file as part of a free software
-// library without restriction.  Specifically, if other files instantiate
-// templates or use macros or inline functions from this file, or you compile
-// this file and link it with other files to produce an executable, this
-// file does not by itself cause the resulting executable to be covered by
-// the GNU General Public License.  This exception does not however
-// invalidate any other reasons why the executable file might be covered by
-// the GNU General Public License.
+// You should have received a copy of the GNU General Public License and
+// a copy of the GCC Runtime Library Exception along with this program;
+// see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see
+// <http://www.gnu.org/licenses/>.
 
 //
 // ISO C++ 14882: 22.2.6.3.2  moneypunct virtual functions
@@ -35,15 +31,17 @@
 
 #include <locale>
 
-_GLIBCXX_BEGIN_NAMESPACE(std)
+namespace std _GLIBCXX_VISIBILITY(default)
+{
+_GLIBCXX_BEGIN_NAMESPACE_VERSION
 
   // Construct and return valid pattern consisting of some combination of:
   // space none symbol sign value
   money_base::pattern
-  money_base::_S_construct_pattern(char, char, char)
+  money_base::_S_construct_pattern(char, char, char) throw()
   { return _S_default_pattern; }
 
-  template<> 
+  template<>
     void
     moneypunct<char, true>::_M_initialize_moneypunct(__c_locale, const char*)
     {
@@ -69,7 +67,7 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
 	_M_data->_M_atoms[__i] = money_base::_S_atoms[__i];
     }
 
-  template<> 
+  template<>
     void
     moneypunct<char, false>::_M_initialize_moneypunct(__c_locale, const char*)
     {
@@ -95,18 +93,18 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
 	_M_data->_M_atoms[__i] = money_base::_S_atoms[__i];
     }
 
-  template<> 
+  template<>
     moneypunct<char, true>::~moneypunct()
     { delete _M_data; }
 
-  template<> 
+  template<>
     moneypunct<char, false>::~moneypunct()
     { delete _M_data; }
 
 #ifdef _GLIBCXX_USE_WCHAR_T
-  template<> 
+  template<>
     void
-    moneypunct<wchar_t, true>::_M_initialize_moneypunct(__c_locale, 
+    moneypunct<wchar_t, true>::_M_initialize_moneypunct(__c_locale,
 							const char*)
     {
       // "C" locale
@@ -120,9 +118,9 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
       _M_data->_M_curr_symbol = L"";
       _M_data->_M_curr_symbol_size = 0;
       _M_data->_M_positive_sign = L"";
-      _M_data->_M_positive_sign_size = 0;      
+      _M_data->_M_positive_sign_size = 0;
       _M_data->_M_negative_sign = L"";
-      _M_data->_M_negative_sign_size = 0;      
+      _M_data->_M_negative_sign_size = 0;
       _M_data->_M_frac_digits = 0;
       _M_data->_M_pos_format = money_base::_S_default_pattern;
       _M_data->_M_neg_format = money_base::_S_default_pattern;
@@ -132,9 +130,9 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
 	  static_cast<wchar_t>(money_base::_S_atoms[__i]);
     }
 
-  template<> 
+  template<>
     void
-    moneypunct<wchar_t, false>::_M_initialize_moneypunct(__c_locale, 
+    moneypunct<wchar_t, false>::_M_initialize_moneypunct(__c_locale,
 							 const char*)
     {
       // "C" locale
@@ -160,13 +158,14 @@ _GLIBCXX_BEGIN_NAMESPACE(std)
 	  static_cast<wchar_t>(money_base::_S_atoms[__i]);
     }
 
-  template<> 
+  template<>
     moneypunct<wchar_t, true>::~moneypunct()
     { delete _M_data; }
 
-  template<> 
+  template<>
     moneypunct<wchar_t, false>::~moneypunct()
     { delete _M_data; }
 #endif
 
-_GLIBCXX_END_NAMESPACE
+_GLIBCXX_END_NAMESPACE_VERSION
+} // namespace
